
module EWB

  class ToHTML

    def initialize( f )
      @f = f

      @nopara = 0
      @kydef = @kyref = 0
    end

    def puts( str = nil )
      if str then
        @f.puts str
      else
        @f.puts
      end
    end

    def print( str )
      @f.print str
    end

    ESC = {
      '&' => '&amp;',
      '<' => '&lt;',
      '>' => '&gt;'
    }

    def esc( str )
      str.gsub( /[&<>]/ ) {|s| ESC[s] }
    end


    def ewb_toplevel( node )
      puts '<html>'
      puts '<head>'
      puts '<title>created by ewb2html</title>'
      puts '</head>'
      puts '<body>'
      yield
      puts '</body>'
      puts '</html>'
    end

    def ewb_i( node )
      puts "<h1>#{node.label}</h1>"
      yield
    end

    def ewb_ii( node )
      puts "<h2>#{node.label}</h2>"
      yield
    end

    def ewb_iii( node )
      puts "<h3>#{node.label}</h3>"
      yield
    end

    def ewb_iiii( node )
      puts "<h4>#{node.label}</h4>"
      yield
    end

    def ewb_iiiii( node )
      puts "<h5>#{node.label}</h5>"
      yield
    end

    def ewb_iiiiii( node )
      puts "<h6>#{node.label}</h6>"
      yield
    end


    def ewb_note( node )
      puts '<h6>NOTE</h6>'
      yield
    end

    def ewb_ky( node )
      puts '<ul>'
      @nopara += 1
      yield
      @nopara -= 1
      puts '</ul>'
    end

    def ewb_read( node )
      puts '<blockquote>'
      yield
      puts '</blockquote>'
    end

    def ewb_lst1( node )
      puts '<blockquote><pre>'
      @nopara += 1
      yield
      @nopara -= 1
      puts '</pre></blockquote>'
    end

    def ewb_lst2( node )
      puts '<pre>'
      @nopara += 1
      yield
      @nopara -= 1
      puts '</pre>'
    end

    def ewb_k1( node )
      @nopara += 1
      puts '<ul>'
      node.each do |n|
        print '<li>'
        n.accept self
      end
      puts '</ul>'
      @nopara -= 1
    end

    def ewb_k2( node )
      @nopara += 1
      puts '<ol>'
      node.each do |n|
        print '<li>'
        n.accept self
      end
      puts '</ol>'
      @nopara -= 1
    end

    def ewb_tabs( node )
      puts '<pre>'   # tmp
      @nopara += 1
      yield
      @nopara -= 1
      puts '</pre>'
    end

    alias ewb_tabm ewb_tabs
    alias ewb_tabl ewb_tabs

    def ewb_ka( node )
      puts '-------------------------'
    end

    alias ewb_kb ewb_ka

    def ewb_cb( node )
      puts '<p><center>'
      @nopara += 1
      yield
      @nopara -= 1
      puts '</center></p>'
    end


    def ewb_l_def( node )
      puts '<p>'
      print "<b>list #{node.ident}: "
      yield
      puts '</b>'
    end

    def ewb_f_def( node )
      puts '<p>'
      print "<b>fig #{node.ident}: "
      yield
      puts '</b>'
    end

    def ewb_l_ref( node )
      print "<list #{node.ident}>"
    end

    def ewb_f_ref( node )
      print "<figure #{node.ident}>"
    end

    def ewb_ky_def( node )
      @kydef += 1
      print %Q_<li><a name="ky#{node.ident}">*#{@kydef}</a> _
      yield
      puts
    end

    def ewb_ky_ref( node )
      @kyref += 1
      print %Q_<a href="\#ky#{node.ident}">*#{@kyref}</a>_
    end


    def ewb_paragraph( node )
      return if @nopara == 0 and node.empty?

      puts '<p>' if @nopara == 0
      yield
      puts '</p>' if @nopara == 0
    end

    def ewb_tt( node )
      print '<code>'
      yield
      print '</code>'
    end

    def ewb_gg( node )
      print '<b>'
      yield
      print '</b>'
    end

    def ewb_g( node )
      print '<em>'
      yield
      print '</em>'
    end

    def ewb_h( node )
      # discard
    end

    def ewb_text( str )
      print esc(str)
    end


    def ewb_label( node )
      # discard
    end

    def ewb_pageref( node )
      print '??'
    end


    def method_missing( name, *args )
      puts name
    end
  
  end


  Visitor = ToHTML

end
