#
# totxt.rb
#
#   Copyright (c) 2001 Minero Aoki <aamine@cd.xdsl.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU General Public License version 2 or later.
#


module EWB

  class ToText

    def initialize( f, w = 74, t = 8 )
      @f = f
      @width = w
      @tabwidth = t

      @w = @save_w = 0
      @pre = 0
      @indent = 0
      @cb = nil

      @sec = 0
      @subsec = 0
    end

    def puts( str = nil )
      if str then
        print_indent
        print str
      end
      @f.puts
      @save_w = @w
      @w = 0
      @head = true
    end

    def print( str )
      print_indent
      if /\t/ === str then
        str << "\t/"       # dummy
        str[0,0] = "/\t"   # dummy
        a = str.split(/\t/)
        a.pop
        a.shift
        s = a.shift
        @f.print s
        @w += s.size
        a.each do |s|
          @w += @tabwidth - (@w % @tabwidth)
          @f.print "\t", s
        end
      else
        @f.print str
        @w += str.size
      end
    end

    def print_indent
      if @head then
        @f.print '  ' * @indent
        @w += @indent * 2
        @head = false
      end
    end


    def ewb_toplevel( node )
      yield
    end

    def ewb_i( node )
      @sec += 1
      @subsec = 0
      puts
      puts
      puts "<<<< Section #{@sec}: #{node.label}>>>>".center(@width)
      puts
      yield
    end

    def ewb_ii( node )
      @subsec += 1
      puts 
      puts "|| #@sec.#@subsec #{node.label} ||"
      puts 
      yield
    end

    def ewb_iii( node )
      puts
      puts "[#{node.label}]"
      puts
      yield
    end

    def ewb_iiii( node )
      puts
      puts " *** #{node.label}"
      yield
    end

    def ewb_iiiii( node )
      puts " ** #{node.label}"
      yield
    end

    def ewb_iiiiii( node )
      puts " * #{node.label}"
      yield
    end


    def ewb_note( node )
      puts
      puts 'NOTE:'
      @indent += 1
      yield
      @indent -= 1
    end

    def ewb_ky( node )
      @indent += 1
      yield
      puts
      @indent -= 1
    end

    def ewb_read( node )
      @indent += 1
      yield
      @indent -= 1
    end

    def ewb_lst1( node )
      @indent += 1
      @pre += 1
      puts
      yield
      puts
      @pre -= 1
      @indent -= 1
    end

    def ewb_lst2( node )
      @pre += 1
      puts
      yield
      puts
      @pre -= 1
    end

    def ewb_k1( node )
      @pre += 1
      @indent += 1
      node.each do |n|
        n.accept self
      end
      @indent -= 1
      @pre -= 1
    end

    def ewb_k2( node )
      @pre += 1
      @indent += 1
      node.each_with_index do |n,i|
        n.accept self
      end
      @indent -= 1
      @pre -= 1
    end

    def ewb_tabs( node )
      @pre += 1
      @indent += 1
      puts
      yield
      puts
      @indent -= 1
      @pre -= 1
    end

    alias ewb_tabm ewb_tabs
    alias ewb_tabl ewb_tabs

    def ewb_ka( node )
      puts '-' * @save_w
    end

    alias ewb_kb ewb_ka

    def ewb_cb( node )
      @cb = []
      yield
      puts @cb.join('').strip.center(@width)
      puts
      @cb = nil
    end


    def ewb_l_def( node )
      puts
      print "list #{node.ident}: "
      yield
      puts
    end

    def ewb_l_ref( node )
      print "[list #{node.ident}]"
    end

    def ewb_f_def( node )
      puts
      print "fig #{node.ident}: "
      yield
      puts
    end

    def ewb_f_ref( node )
      print "[fig #{node.ident}]"
    end

    def ewb_ky_def( node )
      print "*#{node.ident} "
      yield
    end

    def ewb_ky_ref( node )
      print " *#{node.ident} "
    end


    def ewb_paragraph( node )
      return if @pre == 0 and node.empty?

      @head = true
      yield
      puts if @pre == 0
    end

    def ewb_tt( node )
      print ' '
      yield
      print ' '
    end

    def ewb_gg( node )
      yield
    end

    def ewb_g( node )
      yield
    end

    def ewb_h( node )
      # ignore
    end

    def ewb_vt( node )
      print ':'
    end

    def ewb_text( str )
      if @pre > 0 then
        if str[-1,1] == "\n" then
          str.chop!
          puts str
        else
          print str
        end
      elsif @cb then
        @cb.push str
      else
        str.strip.scan(/./) do |s|
          print s
          puts if @w > @width
        end
      end
    end


    def ewb_label( node )
      # discard
    end

    def ewb_pageref( node )
      print '??'
    end


    def method_missing( name, *args )
      @f.puts name
    end
  
  end


  Visitor = ToText

end
